==> makeen-expressionengine-0.0.51.tgz: package/src/operators/operatorsList.ts <==
﻿import { operator } from './operator'
import { binaryOperator } from './binaryOperator'
import { arithmeticOperator } from './arithmeticOperator'
import { boolOperator } from './boolOperator'
import { compareOperator } from './compareOperator'
import { generalOperator } from './generalOperator'
import List from 'typescript-dotnet-commonjs/System/Collections/List'
import { format } from 'typescript-dotnet-commonjs/System/Text/Utility'
import { type } from 'os'
import { utils } from '../compiler/utils'
import { typeMapper } from '../types/typeMapper'
import IEnumerator from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerator'
import IEnumerable from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable'
import DateTime from 'typescript-dotnet-commonjs/System/Time/DateTime'
import { DictionaryObject } from '../utils/dictionaryObject'
export class operatorsList {

    private static _binaryOperators: DictionaryObject;

    public static get binaryOperators(): DictionaryObject {
        if (this._binaryOperators == undefined) {
            this._binaryOperators = new DictionaryObject()
            this.initOperator();
        }
        return this._binaryOperators;
    }

    private static initOperator() {

        let arithmeticOp = new arithmeticOperator();
        arithmeticOp.precedence = 108;
        arithmeticOp.name = "+";
        arithmeticOp.opInvoke = (v: object[]) => {
            if ((v[0] instanceof List && typeof v[1] == 'string') || (typeof v[0] == 'string' && v[1] instanceof List)) {
                let result: List<string> = new List<string>();
                let sourceList: List<object> = (v[0] instanceof List) ? (<List<Object>>v[0]) : (<List<Object>>v[1]);
                let sourceString: string = (typeof v[0] == 'string') ? (<string>v[0]) : (<string><unknown>v[1]);
                for (let obj of sourceList.toArray()) {
                    result.add(format("{0}{1}", obj, sourceString));
                }

                return result;
            }
            else if (v[0] instanceof List || v[1] instanceof List) {
                let result: List<string> = new List<string>();
                let index: number = 0;
                for (var obj of (<List<Object>>v[0]).toArray()) {
                    result.add(format("{0}{1}", obj, (<List<Object>>v[1]).toArray()[index]));
                    index++;
                }

                return result;
            }
            else if (v[0] instanceof DateTime && v[1] instanceof DateTime) {
                return format("{0}{1}", v[0].toJsDate().toString(), v[1].toJsDate().toString());
            }
            else if (v[0] instanceof DateTime || v[1] instanceof DateTime) {
                if (v[0] instanceof DateTime)
                    return format("{0}{1}", v[0] === undefined ? '' : v[0].toJsDate().toString(), v[1] === undefined ? '' : v[1]);
                if (v[1] instanceof DateTime)
                    return format("{0}{1}", v[0] === undefined ? '' : v[0], v[1] === undefined ? '' : v[1].toJsDate().toString());
            }
            else if (typeof v[0] == 'string' || typeof v[1] == 'string') {
                return format("{0}{1}", v[0] === undefined ? '' : v[0], v[1] === undefined ? '' : v[1]);
            }
            else if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '+');
            }

            return undefined;
        };
        let arithmeticOpMinus = new arithmeticOperator();
        arithmeticOpMinus.precedence = 108;
        arithmeticOpMinus.name = "-";
        arithmeticOpMinus.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '-');
            }

            return undefined;
        }
        let arithmeticOpMultiply = new arithmeticOperator();
        arithmeticOpMultiply.precedence = 109;
        arithmeticOpMultiply.name = "*";
        arithmeticOpMultiply.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '*');
            }

            return null;
        }
        let arithmeticOpDivide = new arithmeticOperator();
        arithmeticOpDivide.precedence = 109;
        arithmeticOpDivide.name = "/";
        arithmeticOpDivide.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '/');
            }

            return null;
        }
        let arithmeticOpPercentage = new arithmeticOperator();
        arithmeticOpPercentage.precedence = 109;
        arithmeticOpPercentage.name = "%";
        arithmeticOpPercentage.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '%');
            }

            return null;
        };
        let arithmeticOp104 = new arithmeticOperator();
        arithmeticOp104.precedence = 104;
        arithmeticOp104.name = "^";
        arithmeticOp104.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '^');
            }
            else if (typeof v[0] == 'boolean' && typeof v[1] == 'boolean') {
                //return (typeMapper.toType<boolean>(v[0]) as boolean) ^ (typeMapper.toType<boolean>(v[1]));
            }

            return null;
        }
        arithmeticOp104.canCutOff = undefined;

        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp
            , arithmeticOpMinus, arithmeticOpMultiply, arithmeticOpDivide, arithmeticOpPercentage, arithmeticOp104]);

        let arithmeticOp105 = new boolOperator();
        arithmeticOp105.precedence = 105;
        arithmeticOp105.name = "&";
        arithmeticOp105.opInvoke = (v: Object[]) => {
            let res: boolean = (v[0] as boolean && v[1] as boolean);

            return res;
        }
        arithmeticOp105.canCutOff = undefined;
        // arithmeticOp105.canCutOff =  ( currentTruth:boolean) =>
        // {
        //     return !currentTruth;
        // };
        let arithmeticOp103 = new boolOperator();
        arithmeticOp103.precedence = 103;
        arithmeticOp103.name = "|";
        arithmeticOp103.opInvoke = (v: Object[]) => {
            let res: boolean = (v[0] as boolean || v[1] as boolean);

            return res;
        }
        arithmeticOp105.canCutOff = undefined;
        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp105, arithmeticOp103]);

        let arithmeticOp106 = new compareOperator();
        arithmeticOp106.precedence = 106;
        arithmeticOp106.name = "=";
        arithmeticOp106.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '=');
        }

        let arithmeticOp1061 = new compareOperator();
        arithmeticOp1061.precedence = 106;
        arithmeticOp1061.name = "!";
        arithmeticOp1061.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '!');
        }

        let arithmeticOp1062 = new compareOperator();
        arithmeticOp1062.precedence = 106;
        arithmeticOp1062.name = "~";
        arithmeticOp1062.opInvoke = (v: Object[]) => {
            if (typeof v[0] == 'string' && typeof v[1] == 'string') {
                let strv1: string = v[0] as string;
                let strv2: string = v[1] as string;

                return strv2.indexOf(strv1) > -1;
            }
            if (typeof v[0] == 'number' && typeof v[1] == 'number') {
                let strv1: string = v[0].toString();
                let strv2: string = v[1].toString();

                return strv2.indexOf(strv1) > -1;
            }
            else if (v[1] instanceof List) {
                let ie: IEnumerator<string> = (v[1] as IEnumerable<string>).getEnumerator();

                while (ie.moveNext()) {
                    if (ie.current?.equals(v[0]))
                        return true;
                }
            }

            return false;
        }

        let arithmeticOp1063 = new compareOperator();
        arithmeticOp1063.precedence = 106;
        arithmeticOp1063.name = "#";
        arithmeticOp1063.opInvoke = (v: Object[]) => {
            return v[0] == v[1] || (v[0] != null && v[1] != null && v[1].toString().indexOf(v[0].toString()) >= 0);
        }

        let arithmeticOp107 = new compareOperator();
        arithmeticOp107.precedence = 107;
        arithmeticOp107.name = "<";
        arithmeticOp107.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '<');
        }

        let arithmeticOp1071 = new compareOperator();
        arithmeticOp1071.precedence = 107;
        arithmeticOp1071.name = ">";
        arithmeticOp1071.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '>');
        }

        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp106, arithmeticOp1061, arithmeticOp1062, arithmeticOp1063, arithmeticOp107, arithmeticOp1071]);

        let arithmeticOp100 = new generalOperator();
        arithmeticOp100.precedence = 100;
        arithmeticOp100.name = ":";
        arithmeticOp100.opInvoke = (v: Object[]) => {
            let dict = new DictionaryObject();
            dict.setValue(typeMapper.toType1<string>(v[0]) as string, v[1]);
            return dict;

        }

        operatorsList.addOperator(operatorsList._binaryOperators, arithmeticOp100);
        let arithmeticOp1001 = new generalOperator();
        arithmeticOp1001.precedence = 100;
        arithmeticOp1001.name = "@";
        arithmeticOp1001.opInvoke = (v: Object[]) => {
            let result = new DictionaryObject();
            result.setValue(typeMapper.toType1<string>(v[1]) as string, v[0]);
            return result;

        }
        operatorsList.addOperator(operatorsList._binaryOperators, arithmeticOp1001);

    }

    public static addOperator<T extends operator>(target: DictionaryObject, op: T) {
        target.setValue(op.name as string, op);
    }

    public static addOperators<T extends operator>(target: DictionaryObject, ops: T[]) {
        for (let item of ops) {
            operatorsList.addOperator(target, item);
        }

    }
}

==> makeen-expressionengine-0.0.63.tgz: package/src/operators/operatorsList.ts <==
﻿import { operator } from './operator'
import { binaryOperator } from './binaryOperator'
import { arithmeticOperator } from './arithmeticOperator'
import { boolOperator } from './boolOperator'
import { compareOperator } from './compareOperator'
import { generalOperator } from './generalOperator'
import List from 'typescript-dotnet-commonjs/System/Collections/List'
import { format } from 'typescript-dotnet-commonjs/System/Text/Utility'
import { type } from 'os'
import { utils } from '../compiler/utils'
import { typeMapper } from '../types/typeMapper'
import IEnumerator from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerator'
import IEnumerable from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable'
import DateTime from 'typescript-dotnet-commonjs/System/Time/DateTime'
import { DictionaryObject } from '../utils/dictionaryObject'
export class operatorsList {

    private static _binaryOperators: DictionaryObject;

     static get binaryOperators(): DictionaryObject {
        if (this._binaryOperators == undefined) {
            this._binaryOperators = new DictionaryObject()
            this.initOperator();
        }
        return this._binaryOperators;
    }

    private static initOperator() {

        let arithmeticOp = new arithmeticOperator();
        arithmeticOp.precedence = 108;
        arithmeticOp.name = "+";
        arithmeticOp.opInvoke = (v: object[]) => {
            if ((v[0] instanceof List && typeof v[1] == 'string') || (typeof v[0] == 'string' && v[1] instanceof List)) {
                let result: List<string> = new List<string>();
                let sourceList: List<object> = (v[0] instanceof List) ? (<List<Object>>v[0]) : (<List<Object>>v[1]);
                let sourceString: string = (typeof v[0] == 'string') ? (<string>v[0]) : (<string><unknown>v[1]);
                for (let obj of sourceList.toArray()) {
                    result.add(format("{0}{1}", obj, sourceString));
                }

                return result;
            }
            else if (v[0] instanceof List || v[1] instanceof List) {
                let result: List<string> = new List<string>();
                let index: number = 0;
                for (const obj of (<List<Object>>v[0]).toArray()) {
                    result.add(format("{0}{1}", obj, (<List<Object>>v[1]).toArray()[index]));
                    index++;
                }

                return result;
            }
            else if (v[0] instanceof DateTime && v[1] instanceof DateTime) {
                return format("{0}{1}", v[0].toJsDate().toString(), v[1].toJsDate().toString());
            }
            else if (v[0] instanceof DateTime || v[1] instanceof DateTime) {
                if (v[0] instanceof DateTime)
                    return format("{0}{1}", v[0] === undefined ? '' : v[0].toJsDate().toString(), v[1] === undefined ? '' : v[1]);
                if (v[1] instanceof DateTime)
                    return format("{0}{1}", v[0] === undefined ? '' : v[0], v[1] === undefined ? '' : v[1].toJsDate().toString());
            }
            else if (typeof v[0] == 'string' || typeof v[1] == 'string') {
                return format("{0}{1}", v[0] === undefined ? '' : v[0], v[1] === undefined ? '' : v[1]);
            }
            else if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '+');
            }

            return undefined;
        };
        let arithmeticOpMinus = new arithmeticOperator();
        arithmeticOpMinus.precedence = 108;
        arithmeticOpMinus.name = "-";
        arithmeticOpMinus.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '-');
            }

            return undefined;
        }
        let arithmeticOpMultiply = new arithmeticOperator();
        arithmeticOpMultiply.precedence = 109;
        arithmeticOpMultiply.name = "*";
        arithmeticOpMultiply.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '*');
            }

            return null;
        }
        let arithmeticOpDivide = new arithmeticOperator();
        arithmeticOpDivide.precedence = 109;
        arithmeticOpDivide.name = "/";
        arithmeticOpDivide.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '/');
            }

            return null;
        }
        let arithmeticOpPercentage = new arithmeticOperator();
        arithmeticOpPercentage.precedence = 109;
        arithmeticOpPercentage.name = "%";
        arithmeticOpPercentage.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '%');
            }

            return null;
        };
        let arithmeticOp104 = new arithmeticOperator();
        arithmeticOp104.precedence = 104;
        arithmeticOp104.name = "^";
        arithmeticOp104.opInvoke = (v: object[]) => {
            if (utils.supportsOperators(v[0]) && utils.supportsOperators(v[1])) {
                return utils.toBiggerTypeOperation(v[0], v[1], '^');
            }
            else if (typeof v[0] == 'boolean' && typeof v[1] == 'boolean') {
                //return (typeMapper.toType<boolean>(v[0]) as boolean) ^ (typeMapper.toType<boolean>(v[1]));
            }

            return null;
        }
        arithmeticOp104.canCutOff = undefined;

        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp
            , arithmeticOpMinus, arithmeticOpMultiply, arithmeticOpDivide, arithmeticOpPercentage, arithmeticOp104]);

        let arithmeticOp105 = new boolOperator();
        arithmeticOp105.precedence = 105;
        arithmeticOp105.name = "&";
        arithmeticOp105.opInvoke = (v: Object[]) => {
            let res: boolean = (v[0] as boolean && v[1] as boolean);

            return res;
        }
        arithmeticOp105.canCutOff = undefined;
        // arithmeticOp105.canCutOff =  ( currentTruth:boolean) =>
        // {
        //     return !currentTruth;
        // };
        let arithmeticOp103 = new boolOperator();
        arithmeticOp103.precedence = 103;
        arithmeticOp103.name = "|";
        arithmeticOp103.opInvoke = (v: Object[]) => {
            let res: boolean = (v[0] as boolean || v[1] as boolean);

            return res;
        }
        arithmeticOp105.canCutOff = undefined;
        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp105, arithmeticOp103]);

        let arithmeticOp106 = new compareOperator();
        arithmeticOp106.precedence = 106;
        arithmeticOp106.name = "=";
        arithmeticOp106.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '=');
        }

        let arithmeticOp1061 = new compareOperator();
        arithmeticOp1061.precedence = 106;
        arithmeticOp1061.name = "!";
        arithmeticOp1061.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '!');
        }

        let arithmeticOp1062 = new compareOperator();
        arithmeticOp1062.precedence = 106;
        arithmeticOp1062.name = "~";
        arithmeticOp1062.opInvoke = (v: Object[]) => {
            if (typeof v[0] == 'string' && typeof v[1] == 'string') {
                let strv1: string = v[0] as string;
                let strv2: string = v[1] as string;

                return strv2.indexOf(strv1) > -1;
            }
            if (typeof v[0] == 'number' && typeof v[1] == 'number') {
                let strv1: string = v[0].toString();
                let strv2: string = v[1].toString();

                return strv2.indexOf(strv1) > -1;
            }
            else if (v[1] instanceof List) {
                let ie: IEnumerator<string> = (v[1] as IEnumerable<string>).getEnumerator();

                while (ie.moveNext()) {
                    if (ie.current?.equals(v[0]))
                        return true;
                }
            }

            return false;
        }

        let arithmeticOp1063 = new compareOperator();
        arithmeticOp1063.precedence = 106;
        arithmeticOp1063.name = "#";
        arithmeticOp1063.opInvoke = (v: Object[]) => {
            return v[0] == v[1] || (v[0] != null && v[1] != null && v[1].toString().indexOf(v[0].toString()) >= 0);
        }

        let arithmeticOp107 = new compareOperator();
        arithmeticOp107.precedence = 107;
        arithmeticOp107.name = "<";
        arithmeticOp107.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '<');
        }

        let arithmeticOp1071 = new compareOperator();
        arithmeticOp1071.precedence = 107;
        arithmeticOp1071.name = ">";
        arithmeticOp1071.opInvoke = (v: Object[]) => {
            return utils.compare(v[0], v[1], '>');
        }

        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp106, arithmeticOp1061, arithmeticOp1062, arithmeticOp1063, arithmeticOp107, arithmeticOp1071]);

        let arithmeticOp100 = new generalOperator();
        arithmeticOp100.precedence = 100;
        arithmeticOp100.name = ":";
        arithmeticOp100.opInvoke = (v: Object[]) => {
            let dict = new DictionaryObject();
            dict.setValue(typeMapper.toType1<string>(v[0]) as string, v[1]);
            return dict;

        }

        operatorsList.addOperator(operatorsList._binaryOperators, arithmeticOp100);
        let arithmeticOp1001 = new generalOperator();
        arithmeticOp1001.precedence = 100;
        arithmeticOp1001.name = "@";
        arithmeticOp1001.opInvoke = (v: Object[]) => {
            let result = new DictionaryObject();
            result.setValue(typeMapper.toType1<string>(v[1]) as string, v[0]);
            return result;

        }
        operatorsList.addOperator(operatorsList._binaryOperators, arithmeticOp1001);

    }

     static addOperator<T extends operator>(target: DictionaryObject, op: T) {
        target.setValue(op.name as string, op);
    }

     static addOperators<T extends operator>(target: DictionaryObject, ops: T[]) {
        for (let item of ops) {
            operatorsList.addOperator(target, item);
        }

    }
}
